#include "fss_identify.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_fss_identify_setting_load_
  void fss_identify_setting_load(const f_console_arguments_t arguments, fss_identify_main_t * const main) {

    if (!main) return;

    main->setting.state.step_small = fss_identify_allocation_console_d;

    f_console_parameter_process(arguments, &main->program.parameters, &main->setting.state, 0);

    main->setting.state.step_small = fss_identify_allocation_small_d;

    if (F_status_is_error(main->setting.state.status)) {
      fss_identify_print_error(&main->program.error, macro_fss_identify_f(f_console_parameter_process));

      return;
    }

    main->setting.state.status = fll_program_parameter_process_context_standard(F_true, &main->program);

    if (F_status_is_error(main->setting.state.status)) {
      fss_identify_print_error(&main->program.error, macro_fss_identify_f(fll_program_parameter_process_context_standard));

      return;
    }

    main->setting.state.status = fll_program_parameter_process_verbosity_standard(F_true, &main->program);

    if (F_status_is_error(main->setting.state.status)) {
      fss_identify_print_error(&main->program.error, macro_fss_identify_f(fll_program_parameter_process_verbosity_standard));

      return;
    }

    if (main->program.parameters.array[f_console_standard_parameter_help_e].result & f_console_result_found_e) {
      main->setting.flag |= fss_identify_main_flag_help_d;

      return;
    }

    if (main->program.parameters.array[f_console_standard_parameter_version_e].result & f_console_result_found_e) {
      main->setting.flag |= fss_identify_main_flag_version_d;

      return;
    }

    if (main->program.parameters.array[f_console_standard_parameter_copyright_e].result & f_console_result_found_e) {
      main->setting.flag |= fss_identify_main_flag_copyright_d;

      return;
    }

    f_number_unsigned_t index = 0;

    if (main->program.parameters.array[fss_identify_parameter_content_e].result & f_console_result_found_e) {
      main->setting.flag |= fss_identify_main_flag_content_d;
    }
    else {
      main->setting.flag &= ~fss_identify_main_flag_content_d;
    }

    if (main->program.parameters.array[fss_identify_parameter_object_e].result & f_console_result_found_e) {
      main->setting.flag |= fss_identify_main_flag_object_d;
    }
    else {
      main->setting.flag &= ~fss_identify_main_flag_object_d;
    }

    if (main->program.parameters.array[fss_identify_parameter_total_e].result & f_console_result_found_e) {
      main->setting.flag |= fss_identify_main_flag_total_d;
    }
    else {
      main->setting.flag &= ~fss_identify_main_flag_total_d;
    }

    if (main->program.parameters.array[fss_identify_parameter_line_e].result & f_console_result_value_e) {
      index = main->program.parameters.array[fss_identify_parameter_line_e].values.array[main->program.parameters.array[fss_identify_parameter_line_e].values.used - 1];

      main->setting.state.status = fl_conversion_dynamic_to_unsigned_detect(fl_conversion_data_base_10_c, main->program.parameters.arguments.array[index], &main->setting.line);

      // Negative numbers are not supported.
      if (main->setting.state.status == F_number_negative) {
        main->setting.state.status = F_status_set_error(F_number_negative);
      }

      if (F_status_is_error(main->setting.state.status)) {
        fll_program_print_error_parameter_integer_not(&main->program.error, f_console_symbol_long_normal_s, fss_identify_long_line_s, main->program.parameters.arguments.array[index]);

        return;
      }

      main->setting.flag |= fss_identify_main_flag_line_d;
    }
    else if (main->program.parameters.array[fss_identify_parameter_line_e].result & f_console_result_found_e) {
      main->setting.state.status = F_status_set_error(F_parameter);

      fll_program_print_error_parameter_missing_value(&main->program.error, f_console_symbol_long_normal_s, fss_identify_long_line_s);

      return;
    }
    else {
      main->setting.flag &= ~fss_identify_main_flag_line_d;
    }

    if (main->program.parameters.array[fss_identify_parameter_name_e].result & f_console_result_value_e) {
      main->setting.names.used = 0;

      main->setting.state.status = f_memory_array_increase_by(main->program.parameters.array[fss_identify_parameter_name_e].values.used, sizeof(f_string_dynamic_t), (void **) &main->setting.names.array, &main->setting.names.used, &main->setting.names.size);

      if (F_status_is_error(main->setting.state.status)) {
        fss_identify_print_error(&main->program.error, macro_fss_identify_f(f_memory_array_increase_by));

        return;
      }

      main->setting.names.used = main->program.parameters.array[fss_identify_parameter_name_e].values.used;

      f_number_unsigned_t i = 0;
      f_number_unsigned_t j = 0;
      f_status_t status = F_okay;

      for (; i < main->setting.names.used; ++i) {

        if (fss_identify_signal_check(main)) return;

        index = main->program.parameters.array[fss_identify_parameter_name_e].values.array[i];
        main->setting.names.array[i].used = 0;

        main->setting.state.status = f_string_dynamic_append_nulless(main->program.parameters.arguments.array[index], &main->setting.names.array[i]);

        if (F_status_is_error(main->setting.state.status)) {
          fss_identify_print_error(&main->program.error, macro_fss_identify_f(f_string_dynamic_append_nulless));

          break;
        }

        if (main->setting.names.array[i].used) {
          for (j = 0; j < main->setting.names.array[i].used; ++j) {

            if (fss_identify_signal_check(main)) return;

            main->setting.state.status = f_utf_is_word_dash(main->setting.names.array[i].string + j, main->setting.names.array[i].used, F_true);

            if (F_status_is_error(main->setting.state.status)) {
              fss_identify_print_error(&main->program.error, macro_fss_identify_f(f_utf_is_word_dash));

              return;
            }

            if (main->setting.state.status == F_false) {
              main->setting.state.status = F_status_set_error(F_parameter);

              if (F_status_is_error_not(main->setting.state.status)) {
                status = F_status_set_error(F_parameter);
              }

              fss_identify_print_error_parameter_file_name_invalid(&main->program.error, index, main->setting.names.array[i]);

              break;
            }
          } // for
        }
        else {
          if (F_status_is_error_not(main->setting.state.status)) {
            status = F_status_set_error(F_parameter);
          }

          fss_identify_print_error_parameter_file_name_empty(&main->program.error, index);
        }
      } // for

      if (F_status_is_error(main->setting.state.status)) return;

      if (F_status_is_error(status)) {
        main->setting.state.status = status;

        return;
      }

      main->setting.flag |= fss_identify_main_flag_name_d;
    }
    else if (main->program.parameters.array[fss_identify_parameter_name_e].result & f_console_result_found_e) {
      main->setting.state.status = F_status_set_error(F_parameter);

      fll_program_print_error_parameter_missing_value(&main->program.error, f_console_symbol_long_normal_s, fss_identify_long_name_s);

      return;
    }
    else {
      main->setting.flag &= ~fss_identify_parameter_name_e;
    }

    if (main->program.parameters.remaining.used) {
      main->setting.files.used = 0;

      main->setting.state.status = f_memory_array_increase_by(main->program.parameters.remaining.used, sizeof(f_string_dynamic_t), (void **) &main->setting.files.array, &main->setting.files.used, &main->setting.files.size);

      if (F_status_is_error(main->setting.state.status)) {
        fss_identify_print_error(&main->program.error, macro_fss_identify_f(f_memory_array_increase_by));

        return;
      }

      // Construct the array without allocating any more memory by setting this as a static string (used > 0, size = 0).
      for (f_number_unsigned_t index = 0; main->setting.files.used < main->program.parameters.remaining.used; ) {

        if (fss_identify_signal_check(main)) return;

        index = main->program.parameters.remaining.array[main->setting.files.used];

        main->setting.files.array[main->setting.files.used].string = main->program.parameters.arguments.array[index].string;
        main->setting.files.array[main->setting.files.used].used = main->program.parameters.arguments.array[index].used;
        main->setting.files.array[main->setting.files.used++].size = 0;
      } // for

      main->setting.flag |= fss_identify_main_flag_file_from_d;
    }
    else {
      main->setting.flag &= ~fss_identify_main_flag_file_from_d;
    }

    if (main->program.pipe & fll_program_data_pipe_input_e) {
      main->setting.flag |= fss_identify_main_flag_pipe_d;
    }
    else {
      main->setting.flag &= ~fss_identify_main_flag_pipe_d;
    }

    if (!(main->setting.flag & fss_identify_main_flag_pipe_d | fss_identify_main_flag_file_from_d)) {
      main->setting.state.status = F_status_set_error(F_parameter);

      fss_identify_print_error_no_from(&main->program.error);

      return;
    }

    // Default to both Object and Content when neither are specified.
    if (!(main->setting.flag & (fss_identify_main_flag_object_d | fss_identify_main_flag_content_d | fss_identify_main_flag_total_d))) {
      main->setting.flag |= fss_identify_main_flag_object_d;
      main->setting.flag |= fss_identify_main_flag_content_d;
    }

    if (main->setting.flag & fss_identify_main_flag_total_d) {
      if (main->setting.flag & (fss_identify_main_flag_content_d | fss_identify_main_flag_object_d)) {
        main->setting.state.status = F_status_set_error(F_parameter);

        if (main->program.error.verbosity > f_console_verbosity_quiet_e) {
          fll_program_print_error_parameter_cannot_use_with(&main->program.error, f_console_symbol_long_normal_s, f_console_symbol_long_normal_s, main->setting.flag & fss_identify_main_flag_content_d ? fss_identify_long_content_s : fss_identify_long_object_s, fss_identify_long_total_s);
        }
      }
    }
  }
#endif // _di_fss_identify_setting_load_

#ifdef __cplusplus
} // extern "C"
#endif
